package Program::Statement;

use strict;
use vars qw/$VERSION @ISA @EXPORT/;

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(

);
$VERSION = '0.01';

# Preloaded methods go here.
sub new {
    my $that = shift;
    my $previous = shift;
    my $down = shift;
    my $class = ref($that) || $that;
    my $self = {
	text => '',
	comment => '',
	code => '',
	description => '',	# Not used yet.
	nxt => undef,
	up => undef,
	prev => $previous,
	down => $down
	};
    bless $self, $class;
    return $self;
}

sub add_text {
    my $self = shift;
    $self->{text} .= shift;
}

sub add_comment {
    my $self = shift;
    my $cmt = $_[0];
    # If there are only spaces or tabs in the current statement,
    # then this comment is a line comment for the previous statement.
    # Add it to that one then.

    if ($self->{text} =~ /[^ \t]/) {
	$self->{comment} .= $cmt;
    }
    elsif (ref $self->{prev}) {
	$self->{prev}->add_comment($cmt);
    }
    elsif (ref $self->{down}) {
	$self->{down}->add_comment($cmt);
    }
    else {
	$self->{comment} = $self->{comment} . $cmt;
    }
}

sub add_description {
    my $self = shift;
    my $doc = $_[0];

    # If there are only spaces or tabs in the current statement,
    # then this comment is a line comment for the previous statement.
    # Add it to that one then.

    if ($self->{text} =~ /[^ \t]/) {
	$self->{description} .= $doc;
    }
    elsif (ref $self->{prev}) {
	$self->{prev}->add_description($doc);
    }
    elsif (ref $self->{down}) {
	$self->{down}->add_description($doc);
    }
    else {
	$self->{description} = $self->{description} . $doc;
    }
}

sub add_code {
    my $self = shift;
    my $code = shift;
    $code =~ y/ \t\r\n\f/ /s;
    $code =~ s/^ // if $self->{code} =~ / $/ or !$self->{code};
    $self->{code} .= $code;
}

sub add_quoted_code {
    my $self = shift;
    $self->{code} .= shift;
}

sub next_statement {
    my $self = shift;
    $self->{code} =~ s/ $//;
    if (ref $self->{nxt}) {
	return $self->{nxt};
    }
    elsif ($self->{code} eq '') {
	if (ref($self->{prev}) and $self->{text} !~ /[^ \t]/) {
	    $self->{prev}->add_comment($self->{comment});
	    $self->{prev}->add_text($self->{text});
	    $self->{comment} = '';
	    $self->{text} = '';
	}
	return $self;
    }
    else {
	$self->{nxt} = Program::Statement->new($self, $self->{down});
	return $self->{nxt};
    }
}

sub last_statement {
    my $self = shift;
    $self->{code} =~ s/ $//;
# If the current statement is blank, combine it with the previous
# one.  This doesn't quite seem to be working though.
#    if ($self->{code} eq '' and ref $self->{prev}) {
#	$self->{prev}->add_comment($self->{comment});
#	$self->{prev}->add_text($self->{text});
#	undef $self->{prev}->{nxt};
#    }
    if (!ref($self->{down})) {	# Mismatched brackets... don't give up though.
	$self->{down} = new Program::Statement->new($self, undef);
    }
    if (!ref($self->{down}->{nxt})) { # Create a new path.
	$self->{down}->{nxt} =
	    new Program::Statement->new($self->{down}, $self->{down}->{down});
    }
    return $self->{down}->{nxt};
}

sub next_statement_in_code {
    my $self = shift;
    # If this statement starts a block, return the first statement
    # in that block.
    return $self->{'up'} if ref $self->{'up'};
    return $self->{'nxt'} if ref $self->{'nxt'};
    return $self->{'down'}->{'nxt'}
	if ref $self->{'down'} and ref $self->{'down'}->{'nxt'};
    return undef;
}

sub up {
    my $self = shift;
    # If we aren't able to go up, create a way.
    $self->{up} = Program::Statement->new(undef, $self)
        unless ref($self->{up});
    return $self->{up};
}

# Get the next statement.
sub get_next {return $_[0]->{'nxt'}}

# Get the statement a level up.
sub get_up {return $_[0]->{'up'}}

# Get the statement a level down.
sub get_down {return $_[0]->{'down'}}

# Get the statement previous.
sub get_prev {return $_[0]->{'prev'}}

sub get_code {
    my $self = shift;
    return $self->{code};
}

sub get_comment {
    my $self = shift;
    return $self->{comment};
}

sub get_description {
    my $self = shift;
    return $self->{description};
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Program::Statement

=head1 SYNOPSIS

Program::Statement 

=head1 DESCRIPTION

Program::Statement

=head1 BUGS

The documentation is quite incomplete.

=head1 AUTHOR

James Yolkowski

=head1 SEE ALSO

perl(1).

=cut
